@@ -1,10 +1,11 @@
 # non-convex fair regression from Komiyama et al. (2018).
-nclm = function(response, predictors, sensitive, unfairness, covfun, lambda) {
+nclm = function(response, predictors, sensitive, unfairness, covfun, lambda = 0,
+         save.auxiliary = FALSE) {
 
   fitted = two.stage.regression(model = "nclm", response = response,
              predictors = predictors, sensitive = sensitive,
-             unfairness = unfairness,
-             covfun = covfun, lambda = lambda)
+             unfairness = unfairness, definition = "sp-komiyama",
+             covfun = covfun, lambda = lambda, save.auxiliary = save.auxiliary)
 
   # save the function call for the print() method.
   fitted$main$call = match.call()
@@ -13,94 +14,6 @@ nclm = function(response, predictors, sensitive, unfairness, covfun, lambda) {
 
 }#NCLM
 
-two.stage.regression = function(model, response, predictors, sensitive,
-    unfairness, covfun, lambda) {
-
-  # check the model to be fitted.
-  check.label(model, fair.models, "model")
-  # check the arguments common to all methods.
-  response = check.response(response)
-  predictors = check.data(predictors, nobs = length(response), varletter = "X")
-  sensitive = check.data(sensitive, nobs = length(response), varletter = "S")
-  check.fairness.level(unfairness)
-
-  # check method-specific extra arguments.
-  if (model == "nclm") {
-
-    covfun = check.covariance.function(covfun)
-    if (missing(lambda))
-      lambda = 0
-    else if (!is.non.negative(lambda))
-      stop('lambda should be a non-negative number.')
-
-  }#THEN
-  else if (model == "frrm") {
-
-    # nothing to do.
-
-  }#THEN
-
-  # save some information on the data, to be used e.g. in predict().
-  predictors.info = get.data.info(predictors)
-  sensitive.info = get.data.info(sensitive)
-
-  # encode factors with constrasts, but:
-  # 1) remove the intercept from the predictors (it will be added later in the
-  #      main model);
-  # 2) remove the intercept from the sensitive attributes (it will be added
-  #      later in the auxiliary model).
-  predictors = design.matrix(predictors, intercept = FALSE)
-  sensitive = design.matrix(sensitive, intercept = FALSE)
-  # regress the predictors against the sensitive attributes to contruct U, the
-  # matrix of non-sensitive variables.
-  auxiliary.model = lm(predictors ~ ., data = data.frame(sensitive))
-  U = predictors - fitted(auxiliary.model)
-  # remove the intercept from the sensitive attributes, it is no longer needed.
-  sensitive = sensitive[, colnames(sensitive) != "(Intercept)", drop = FALSE]
-
-  # check that there are no duplicate names.
-  duplicate.names = intersect(colnames(U), colnames(sensitive))
-  if (length(duplicate.names) != 0)
-    stop("duplicate variable names: ", paste(duplicate.names, collapse = " "), ".")
-
-  if (model == "nclm") {
-
-    if (unfairness == 0) {
-
-      fit = nclm.zero.sensitive(y = response, S = sensitive, U = U,
-              covfun = covfun, lambda = lambda)
-
-    }#THEN
-    else {
-
-      fit = nclm.optiSolve(y = response, S = sensitive, U = U,
-              epsilon = unfairness, covfun = covfun, lambda = lambda)
-
-    }#ELSE
-
-  }#THEN
-  else if (model == "frrm") {
-
-    fit = frrm.glmnet(y = response, S = sensitive, U = U,
-            unfairness = unfairness)
-
-  }#THEN
-
-  # collect all the information about the model and the input data.
-  retval = list(
-    auxiliary = list(call = auxiliary.model$call,
-                     coefficients = coef(auxiliary.model),
-                     fitted.value = fitted(auxiliary.model),
-                     residuals = residuals(auxiliary.model)),
-    main = c(call = NA, fit, bound = unfairness, covfun = covfun,
-             lambda = lambda),
-    data = list(predictors = predictors.info, sensitive = sensitive.info)
-  )
-
-  return(structure(retval, class = c(model, "fair.model")))
-
-}#TWO.STAGE.REGRESSION
-
 # particular case, excluding sensitive attributes.
 nclm.zero.sensitive = function(y, S, U, covfun, lambda) {
 
@@ -146,19 +59,39 @@ nclm.zero.sensitive = function(y, S, U, covfun, lambda) {
   # fitted values and residuals.
   fitted = as.vector(coefs["(Intercept)"] + U %*% coefs[colnames(U)])
   resid = as.vector(y - fitted)
-  # proportions of response variance explained by S and U.
+  # proportions of response variance explained by S and U: scale by var(y)
+  # instead of using the "scaled:scale" attribute because scale() is often
+  # called with a hard-coded scale factor of 1.
   r2.S = 0
-  r2.U = var(fitted) / attr(ys, "scaled:scale")^2
+  r2.U = var(fitted) / var(y)
 
   # make sure the coefficients are returned in the same order as in
   # nclm.optiSolve().
   coefs = c(coefs["(Intercept)"], structure(rep(0, nS), names = colnames(S)),
             coefs[colnames(Us)])
-  # mark coefficients corresponding to sensitive variables.
+  # mark coefficients corresponding to sensitive attributes.
   attr(coefs, "sensitive") = names(coefs) %in% colnames(S)
 
-  return(list(coefficients = coefs, residuals = resid, fitted.values = fitted,
-           r.squared.S = r2.S, r.squared.U = r2.U))
+  return(list(main = list(
+                coefficients = coefs,
+                residuals = resid,
+                fitted.values = fitted,
+                r.squared.S = r2.S,
+                r.squared.U = r2.U,
+                r2.statistical.parity = r2.S / (r2.S + r2.U),
+                arguments = list(
+                  lambda = lambda,
+                  covfun = covfun
+              )),
+         fairness = list(
+                definition = "sp-komiyama",
+                value = r2.S / (r2.S + r2.U),
+                bound = 0,
+                other = list(
+                  r.squared.S = r2.S,
+                  r.squared.U = r2.U
+                )
+              )))
 
 }#NCLM.ZERO.SENSITIVE
 
@@ -240,19 +173,39 @@ nclm.optiSolve = function(y, S, U, epsilon, covfun, lambda) {
     as.vector(attr(ys, "scaled:center") -
               attr(Ss, "scaled:center") %*% coefs[colnames(S)] -
               attr(Us, "scaled:center") %*% coefs[colnames(U)])
-  # mark coefficients corresponding to sensitive variables.
+  # mark coefficients corresponding to sensitive attributes.
   attr(coefs, "sensitive") = names(coefs) %in% colnames(S)
   # fitted values and residuals.
   fitted.U = U %*% coefs[colnames(U)]
   fitted.S = S %*% coefs[colnames(S)]
   fitted = as.vector(coefs["(Intercept)"] + fitted.S + fitted.U)
   resid = as.vector(y - fitted)
-  # proportions of response variance explained by S and U.
-  r2.S = var(fitted.S) / attr(ys, "scaled:scale")^2
-  r2.U =  var(fitted.U) / attr(ys, "scaled:scale")^2
+  # proportions of response variance explained by S and U: scale by var(y)
+  # instead of using the "scaled:scale" attribute because scale() is often
+  # called with a hard-coded scale factor of 1.
+  r2.S = var(fitted.S) / var(y)
+  r2.U =  var(fitted.U) / var(y)
 
-  return(list(coefficients = coefs, residuals = resid, fitted.values = fitted,
-           r.squared.S = r2.S, r.squared.U = r2.U))
+  return(list(main = list(
+                coefficients = coefs,
+                residuals = resid,
+                fitted.values = fitted,
+                r.squared.S = r2.S,
+                r.squared.U = r2.U,
+                r2.statistical.parity = r2.S / (r2.S + r2.U),
+                arguments = list(
+                  lambda = lambda,
+                  covfun = covfun
+              )),
+         fairness = list(
+                definition = "sp-komiyama",
+                value = r2.S / (r2.S + r2.U),
+                bound = epsilon,
+                other = list(
+                  r.squared.S = r2.S,
+                  r.squared.U = r2.U
+                )
+              )))
 
 }#NCLM.OPTISOLVE
 
