@@ -1,22 +1,39 @@
 
 # check the response variable.
-check.response = function(response, min.nobs = 2) {
+check.response = function(response, type, min.nobs = 2) {
 
-  if (missing(response) || !is.real.vector(response))
-    stop("'response' should be a numeric vector.")
+  if (missing(response))
+    stop("'response' is missing.")
+
+  if (type == "continuous") {
+
+    if (!is.real.vector(response))
+      stop("'response' should be a numeric vector.")
 
   # make sure the response is a vector.
   response = as.vector(response)
 
+    # do not allow a zero-variance response if later we try to standardize it.
+    if (min.nobs > 1)
+      if (var(response) < sqrt(.Machine$double.eps))
+        stop("'response' has variance zero, it cannot be standardized.")
+
+  }#THEN
+  else if (type == "binary") {
+
+    if (!is.factor(response) || (nlevels(response) != 2))
+      stop("'response' should be a factor with two levels.")
+
+  }#THEN
+
+  # no missing values in the response.
+  if (anyNA(response))
+    stop("'response' contains missing values.")
+
   # check the minimum sample size.
   if (length(response) < min.nobs)
     stop("'response' should contain at least ", min.nobs, " observations.")
 
-  # do not allow a zero-variance response if later we try to standardize it.
-  if (min.nobs > 1)
-    if (var(response) < sqrt(.Machine$double.eps))
-      stop("'response' has variance zero, it cannot be standardized.")
-
   return(response)
 
 }#CHECK.RESPONSE
